%{
@Name:ai_continuous.m
@Author: Smacq
@Copyright: Smacq
@Date: 2025/02/08
@Brief: 
The Ai functions of USB-5000, continuous acquisition.
Acquisition Ai0 channel 100 data point of 10 times.
You can loot out user manual know more about functions or Device info.

USB-5000 only connect USB execution result:
MATLAB is 64-bit.
Dll load success.
Device number: 1
     0
Device index: 0 Device serial number: 66666666 Device model: 5722
     0
...
     0
Remaining buffer: 2
Ai0 100 data points:
arrange 1 to 8
4.0071    4.0055    4.0036    4.0014    3.9989    3.9977    3.9971    3.9967
...  
arrange 97 to 100
4.0089    4.0077    4.0080    4.0077
     0
     0
     0

mex -setup
MEX setup 'Microsoft Visual C++ 2022 (C)' to compile in C++.

To choose a different language, please select a command from the following options:
mex -setup C++ 
mex -setup FORTRAN

MEX setup 'Microsoft Visual C++ 2022' to compile in C++.
%}

clear
clc
% Check MATLAB bit is 32 or 64, load diffrent lib file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\USB5000.dll';  
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\USB5000.dll';  
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\USB5000.h';

try
    loadlibrary(dll_file_path,h_file_path);
    if libisloaded('USB5000')
        fprintf('Dll load success.\n')
        %libfunctions USB5000 -full
    end
catch
    fprintf('Dll load Fail pleaes check matlab bit and lib file path.\n')
    dll_file_path
    h_file_path
end

% Initialize the array for storing the acquired data.
% Usually the array size need >= acquisition data points size.
% Data type is single (float32 in Python, equivalent in Matlab), it is not modifiable.
% The data points saved to Ai_data_array arrange from opened Ai channel.
Ai_data_array = zeros(1, 2400);

%Define pointer to Ai_data_array type is singlePtr
Ai_data_array_ptr = libpointer('singlePtr',Ai_data_array);

% Device index, starting from 0. Assume only one device is operated here.
DeviceIndex = int32(0);

% Ai voltage range:
% 10 = -10V to +10V and so on
% Voltage range param support 10/5
VoltageRange = single(10);

% Ai channel sample mode:
% 0=Continuous acquisition or 1=Oneshot, param support 0/1
SamplingMode = uint8(0);

% Acquisition amount of data:
% Each opened Ai channel will get this amount of data.
% Suggest value within sampling rate 10%~20%.
NumOfDataPoints = uint32(100);

% Analog input channel number. The channel number in the device model starts from 0.
ChannelNumber = uint8(0);

% Values representing the on/off state of the channel.
ChannelOn = uint8(1);
ChannelOff = uint8(0);

% Ai Sampling period, unit ns, steps 10ns:
% USB-5000 is asynchronous DAQ, single channel max rate is 1M=1000.
% Multichannel each channel share out samplerate, 1M/channel.
% sample rate and period:
% 100k = 10000
% 10k  = 100000
% 1k   = 1000000
% 100  = 10000000
% 10   = 100000000
% 1    = 1000000000
SamplingPeriod = uint32(1000000);

% Ai acquisition trig source:
% 0=SetUSB3AiSoftTrig(), 1=Ext_Trig_In, 2~5=Din_0~3, 
% 7=with DI, 8=with DO, 9~12=with AO~3, 255=GlobalSoftTrig
% param support 0/1/2/3/4/5/7/8/9/10/11/12/255
TriggerSource = uint8(0);

% Ai acquisition clock source:
% 0=SetUSB3AiSampleRate(), 1=Conv_In, 3=Di, 4=DO, 5~8=AO~3
% param support 0/1/3/4/5/6/7/8
ClockSource = uint8(0);

% Timeout unit ms:
% Suggest Timeout>(Num*Channel_count/Samplerate)*1000
% Avoid USB3GetAi() func timeout, if timeout return -7.
TimeoutValue = int32(200);


% Find the number of connected DAQ devices.
DeviceNumber = calllib('USB5000', 'FindUSB5DAQ');
disp(['Device number: ', num2str(DeviceNumber)]);

% Open the specified device and get the return error code (assume 0 means success).
ErrorCode = calllib('USB5000', 'USB5OpenDevice', DeviceIndex);
disp(ErrorCode);

% Get and display the device serial number and device model.
DeviceSerialNumber = calllib('USB5000', 'USB5ReadDevcieSN', DeviceIndex);
DeviceModel = calllib('USB5000', 'USB5ReadDevcieModel', DeviceIndex);
disp(['Device index: ', num2str(DeviceIndex), ' Device serial number: ', num2str(DeviceSerialNumber), ' Device model: ', num2str(DeviceModel)]);

% Turn off the default opened Ai channels from 0 to 8 (assume these channels are default opened).
for Channel = 0:7
    ErrorCode = calllib('USB5000', 'SetUSB5AiChanSel', DeviceIndex, Channel, ChannelOff);
    if ErrorCode ~= 0
        disp(ErrorCode);
    end
end

% Turn on and set the input voltage range for Ai channel 0.
for Channel = 0:0
    ErrorCode = calllib('USB5000', 'SetUSB5AiChanSel', DeviceIndex, Channel, ChannelOn);
    disp(ErrorCode);
    ErrorCode = calllib('USB5000', 'SetUSB5AiRange', DeviceIndex, Channel, VoltageRange);
    disp(ErrorCode);
end

% Set the analog input sampling mode of the device.
ErrorCode = calllib('USB5000', 'SetUSB5AiSampleMode', DeviceIndex, SamplingMode);
disp(ErrorCode);

% Set the sampling rate of the device.
ErrorCode = calllib('USB5000', 'SetUSB5AiSampleRate', DeviceIndex, SamplingPeriod);
disp(ErrorCode);

% Set the clock source of the device.
ErrorCode = calllib('USB5000', 'SetUSB5AiConvSource', DeviceIndex, ClockSource);
disp(ErrorCode);

% Clear the AI FIFO (buffer).
ErrorCode = calllib('USB5000', 'SetUSB5ClrAiFifo', DeviceIndex);
disp(ErrorCode);

% Set the trigger source of the device.
ErrorCode = calllib('USB5000', 'SetUSB5AiTrigSource', DeviceIndex, TriggerSource);
disp(ErrorCode);

% Enable the analog input software trigger acquisition.
ErrorCode = calllib('USB5000', 'SetUSB5AiSoftTrig', DeviceIndex);
disp(ErrorCode);

% Continuous acquisition is executed 10 times. Get the data and process it.
for Iteration = 1:10
    RemainingBuffer = calllib('USB5000', 'USB5GetAi', DeviceIndex, NumOfDataPoints, Ai_data_array_ptr,TimeoutValue);
    %{
        remain_buffer range 0~2000000
        USB5GetAi() return the remaining Device FIFO buffer value.
        FIFO buffer 100 data points save to Ai_data_array_pt.
        Channel_0 100 data points is Ai_data_array[0:100]
        If open channel 0~1, Channel_1 100 data points is Ai_data_array[100:200] and so on.
        If remain_buffer increase you should run USB5GetAi() more quick, let remain_buffer steady.
    %}
    disp(['Remaining buffer: ', num2str(RemainingBuffer)]);
    % Display the 100 data points acquired from Ai0 channel.
    disp(['Ai0 100 data points:']);
    Ai_data_array = Ai_data_array_ptr.value;
    disp(Ai_data_array(1:100)); 
    % You can add code here to further process the acquired data each time.
end

% Disable the analog input software trigger acquisition and clear the trigger flag.
ErrorCode = calllib('USB5000', 'SetUSB5ClrAiTrigger', DeviceIndex);
disp(ErrorCode);

% Clear the AI FIFO (buffer) again.
ErrorCode = calllib('USB5000', 'SetUSB5ClrAiFifo', DeviceIndex);
disp(ErrorCode);

% Close the device.
ErrorCode = calllib('USB5000', 'USB5CloseDevice', DeviceIndex);
disp(ErrorCode);

% Unload the dynamic link library.
unloadlibrary('USB5000');