%{
@Name:do_immediately.m
@Author: Smacq
@Copyright: Smacq
@Date: 2025/02/08
@Brief:
The Do functions of USB-5000, digit output.
Set Do output HIGH/LOW, 0V is LOW(default), 3.3V HIGH.
DO channel is different for modul, Do_2 or Do_4 or Do_8.
You can look at user manual know more about functions or Device info.

USB-5000(DO_8) only connect USB execution result:
MATLAB is 64-bit.
Dll load success.
     0
     0
Now Do7-0 is all HIGH
     0
Now Do7-0 is all LOW
     0
Now Do7-0 is all HIGH
     0
Now Do7-0 is all LOW
     0
Now Do7-0 is all HIGH
     0
Now Do7-0 is all LOW
     0
Now DO 0 is 1
     0
Now DO 0 is 0
     0
Now DO 0 is 1
     0
Now DO 0 is 0
     0
Now DO 0 is 1
     0
Now DO 0 is 0
     0
%}

% Clear workspace and command window
clear;
clc;

% Check MATLAB bit is 32 or 64, load different lib file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\USB5000.dll';  % If MATLAB is 64-bit, use this DLL path
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\USB5000.dll';  % If MATLAB is 32-bit, use this DLL path
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\USB5000.h';

try
    loadlibrary(dll_file_path, h_file_path);
    if libisloaded('USB5000')
        fprintf('Dll load success.\n');
    end
catch
    fprintf('Dll load Fail please check matlab bit and lib file path.\n');
end

% Device index starts from zero
DeviceIndex = int32(0);

% For all select digital output channel:
% 0xFF(1111 1111) Do7~Do0 is select all digital output channel.
All_Do_Channel = uint32(0xFF);

% For all Do output Do_Value HIGH is 1 or LOW is 0:
% Do_channel_4:0~0xF(1111) Do3~Do0, Do_channel_8:0~0xFF(1111 1111) Do7~Do0.
All_Do_Value = uint32(0xFF);

% For select single digital output channel:
% Do_channel_4:0~3 -> Do3~Do0, Do_channel_8:0~7 -> Do7~Do0.
single_Do_Channel = uint32(0);

% single Do output Do_Value HIGH is 1:
single_Do_Value_HIGH = uint32(1);

% single Do output Do_Value LOW is 0:
single_Do_Value_LOW = uint32(0);

% Open specific device and print return value.
ErrorCode = calllib('USB5000', 'USB5OpenDevice', DeviceIndex);
disp(ErrorCode);

% Set All Do Pin output HIGH and then LOW, interval of 1 second.
for i = 1:3
    % Set Do Pin output all HIGH.
    ErrorCode = calllib('USB5000', 'SetUSB5DoImmediately', DeviceIndex, All_Do_Channel, All_Do_Value);
    disp(ErrorCode);
    if ErrorCode == 0
        fprintf('Now Do7-0 is all HIGH\n');
    end
    % Necessary delay for easy observation
    pause(1);
    % Set Do Pin output all LOW.
    ErrorCode = calllib('USB5000', 'SetUSB5DoImmediately', DeviceIndex, All_Do_Channel, 0);
    disp(ErrorCode);
    if ErrorCode == 0
        fprintf('Now Do7-0 is all LOW\n');
    end
    % Necessary delay for easy observation
    pause(1);
end

% Set single DO_0 output HIGH/LOW, interval of 1 second.
for i = 1:3
    ErrorCode = calllib('USB5000', 'SetUSB5DoImmediately', DeviceIndex, single_Do_Channel, single_Do_Value_HIGH);
    disp(ErrorCode);
    if ErrorCode == 0
        fprintf('Now DO %d is %d\n', single_Do_Channel, single_Do_Value_HIGH);
    end
    pause(1);
    ErrorCode = calllib('USB5000', 'SetUSB5DoImmediately', DeviceIndex, single_Do_Channel, single_Do_Value_LOW);
    disp(ErrorCode);
    if ErrorCode == 0
        fprintf('Now DO %d is %d\n', single_Do_Channel, single_Do_Value_LOW);
    end
    pause(1);
end

% Close Device
ErrorCode = calllib('USB5000', 'USB5CloseDevice', DeviceIndex);
disp(ErrorCode);

% Unload the dynamic link library
unloadlibrary('USB5000');