%{
@Name:do_waveform_pc_infinite.m
@Author: Smacq
@Copyright: Smacq
@Date: 2025/02/08
@Brief:
The Do functions of USB-5000, 2 type digit waveform output.
Set Do output HIGH/LOW or Flowing, 0V is LOW(default), 3.3V HIGH.
DO channel is different for modul, Do_2 or Do_4 or Do_8.
You can look at user manual know more about functions or Device info.

USB-5000(DO_8) only connect USB execution result (Matlab version, similar concept as Python example):
MATLAB is 64-bit.
Dll load success.
     0
...
     0
Do0~Do7 Flowing HIGH Interval of 1s, Please Enter to new wave.
     0
     0
     0
     0
Do0~Do7 HiHG/LOW Interval of 1s, Please Enter to stop Do output.
     0
     0
     0
%}

% Clear workspace and command window
clear;
clc;

% Check MATLAB bit is 32 or 64, load different lib file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\USB5000.dll';  % If MATLAB is 64-bit, use this DLL path
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\USB5000.dll';  % If MATLAB is 32-bit, use this DLL path
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\USB5000.h';

try
    loadlibrary(dll_file_path, h_file_path);
    if libisloaded('USB5000')
        fprintf('Dll load success.\n');
    end
catch
    fprintf('Dll load Fail please check matlab bit and lib file path.\n');
end

% Init Do_data_array
Do_data_array = uint32([1, 2, 4, 8, 16, 32, 64, 128]);
% Do_data_array = uint32([0xF, 0, 0xFF, 0, 0xF0, 0]);
% Initialize the array for storing the digital output waveform data.
% Data type is uint32, it is not modifiable.
% Do data point 0 is LOW, 1 is HIGH.
% For example:
% Do_channel_4:0xF(1111) Do3~Do0 is HIGH
% Do_channel_8:0xFF(1111 1111) Do7~Do0 is HIGH.
% Do_channel_4:0x3(0011) Do3~Do2 is LOW,Do1~Do0 is HIGH.
% Do_channel_8:0x88(1000 1000) Do7 and Do3 is HIGH, other Do is LOW.

% Define pointer to Do_data_array type is uint32Ptr
Do_data_array_ptr = libpointer('uint32Ptr', Do_data_array);

% Device index starts from zero
DeviceIndex = int32(0);

% select digital output wave channel
% Do_channel_4:0(0000)~0xF(1111) Do3~Do0
% Do_channel_8:0(0000 0000)~0xFF(1111 1111) Do7~Do0
% select Do channel for example:
% Do_0: 0x1(0001), Do_0 and Do_1: 0x3(0011), Do_3 and Do_7: 0x88(1000 1000)
DoWaveChannel = uint32(0xFF);

% Do channel sample mode:
% 0=continuous non-loop output to computer buffer
% 1=FIFO output loop decide by SetUSB5DoCycle()
DoSampleMode = uint8(1);

% Do Sampling period, unit ns, steps 10ns:
% USB-5000 is synchronous Do, max rate is 10M=100.
% sample rate and period:
% 1M   = 1000
% 100k = 10000
% 10k  = 100000
% 1k   = 1000000
% 100  = 10000000
% 10   = 100000000
% 1    = 1000000000
DoSampleRate = uint32(1000000000);

% Do acquisition trig source:
% 0=SetUSB5AiSoftTrig(), 1=Ext_Trig_In, 2~5=Din_0~3, 
% 6=with AI, 7=with DI, 9~12=with AO~3, 255=GlobalSoftTrig
% param support 0/1/2/3/4/5/6/7/9/11/12/255
DoTrigSource = uint8(0);

% Do acquisition clock source:
% 0=SetUSB5DoSampleRate(), 1=Conv_In, 2=Ai, 3=Di, 5~8=AO~3
% param support 0/1/2/3/5/6/7/8
DoConvSource = uint8(0);

% Set waveform output cycle
% 0=infinite loop output
% 1=1 times, 2= 2 times and so on.
% param support 0~4294967295
DoCycle = uint32(0);

% Download to Do FIFO data number
Num = uint32(length(Do_data_array));

% Open specific device and print return value.
ErrorCode = calllib('USB5000', 'USB5OpenDevice', DeviceIndex);
disp(ErrorCode);

% Disable Do software trig, clear flag
ErrorCode = calllib('USB5000', 'SetUSB5ClrDoTrigger', DeviceIndex);
disp(ErrorCode);

% Clear FIFO
ErrorCode = calllib('USB5000', 'SetUSB5ClrDoFifo', DeviceIndex);
disp(ErrorCode);

% Set Device Do sample mode
ErrorCode = calllib('USB5000', 'SetUSB5DoSampleMode', DeviceIndex, DoSampleMode);
disp(ErrorCode);

% Set Device sample rate
ErrorCode = calllib('USB5000', 'SetUSB5DoSampleRate', DeviceIndex, DoSampleRate);
disp(ErrorCode);

% Set digital output wave channel
ErrorCode = calllib('USB5000', 'SetUSB5DoWaveCtrl', DeviceIndex, DoWaveChannel);
disp(ErrorCode);

% Set digital output cycle infinite loop
ErrorCode = calllib('USB5000', 'SetUSB5DoCycle', DeviceIndex, DoCycle);
disp(ErrorCode);

% Set Device trig source
ErrorCode = calllib('USB5000', 'SetUSB5DoTrigSource', DeviceIndex, DoTrigSource);
disp(ErrorCode);

% Set Device clock source
ErrorCode = calllib('USB5000', 'SetUSB5DoConvSource', DeviceIndex, DoConvSource);
disp(ErrorCode);

% Download array data to Do FIFO
ErrorCode = calllib('USB5000', 'SetUSB5DoDataFifo', DeviceIndex, Do_data_array_ptr, Num);
disp(ErrorCode);

% Enable Do software trig start DO output wave
ErrorCode = calllib('USB5000', 'SetUSB5DoSoftTrig', DeviceIndex);
disp(ErrorCode);

% Pause and wait for user input to continue to new wave
input('Do0~Do7 Flowing HIGH Interval of 1s, Please Enter to new wave.');

% Disable Do software trig, clear flag
ErrorCode = calllib('USB5000', 'SetUSB5ClrDoTrigger', DeviceIndex);
disp(ErrorCode);

% Clear FIFO
ErrorCode = calllib('USB5000', 'SetUSB5ClrDoFifo', DeviceIndex);
disp(ErrorCode);

% Update new wave
Do_data_array = uint32([0, 255]);
% Download to Do FIFO data number
Num = uint32(length(Do_data_array));
% Define pointer to Do_data_array type is uint32Ptr
Do_data_array_ptr = libpointer('uint32Ptr', Do_data_array);
% Download array data to Do FIFO
ErrorCode = calllib('USB5000', 'SetUSB5DoDataFifo', DeviceIndex, Do_data_array_ptr, Num);
disp(ErrorCode);

% Enable Do software trig start DO output wave
ErrorCode = calllib('USB5000', 'SetUSB5DoSoftTrig', DeviceIndex);
disp(ErrorCode);

% Pause and wait for user input to stop Do output
input('Do0~Do7 HiHG/LOW Interval of 1s, Please Enter to stop Do output.');

% Disable Do software trig, clear flag
ErrorCode = calllib('USB5000', 'SetUSB5ClrDoTrigger', DeviceIndex);
disp(ErrorCode);

% Clear FIFO
ErrorCode = calllib('USB5000', 'SetUSB5ClrDoFifo', DeviceIndex);
disp(ErrorCode);

% Close Device and stop Do wave output
ErrorCode = calllib('USB5000', 'USB5CloseDevice', DeviceIndex);
disp(ErrorCode);

% Unload the dynamic link library
unloadlibrary('USB5000');